//	CrystalFlightGPUDefinitions.h
//
//	© 2023 by Jeff Weeks
//	See TermsOfUse.txt

#ifndef CrystalFlightGPUDefinitions_h
#define CrystalFlightGPUDefinitions_h

#include <simd/simd.h>
#include "../../../Shared/GeometryGamesFauxSimd.h"	//	Metal files need explicit path


enum
{
	VertexAttributePosition	= 0,
	VertexAttributeNormal	= 1,
	VertexAttributeColor	= 2
};

//	For CrystalFlightVertexFunction
enum
{
	BufferIndexVertexAttributes		= 0,
	BufferIndexUniforms				= 1,
	BufferIndexTiling				= 2
};

//	For CrystalFlightTileCullingFunction
enum
{
	BufferIndexTilingParameters		= 0,
	BufferIndexCulledTiling			= 1
};

//	For CrystalFlightTilingArrayCompactionFunction
enum
{
	BufferIndexCompactableTiling	= 0,
	BufferIndexLoopCount			= 1,
	BufferIndexIndirectArguments	= 2
};

//	For CrystalFlightTransformationAndLightingFunction
enum
{
	BufferIndexRotation				= 0,
	BufferIndexTranslation			= 1,
	BufferIndexUnlitVertices		= 2,
	BufferIndexLitVertices			= 3,
	BufferIndexLevelOfDetailColor	= 4	//	used only with gColorWithLevelOfDetail
};


typedef struct
{
	simd_float4x4	itsProjectionMatrix;

	__fp16	itsHorizonRadius,	//	in ångströms
			itsFogBegin,		//	in ångströms
			itsFogRangeInverse;	//	in ångströms⁻¹

	faux_simd_half3	itsLevelOfDetailColor[4];	//	used only with gColorWithLevelOfDetail

} CrystalFlightUniformData;


typedef struct
{
	simd_float3		pos;	//	position (x,y,z)
	faux_simd_half3	nor;	//	unit-length normal vector (nx, ny, nz)
	faux_simd_half3	col;	//	linear extended-range sRGB

} CrystalFlightUnlitVertex;

typedef struct
{
	simd_float3		pos;	//	position (x,y,z)
	faux_simd_half3	col;	//	linear extended-range sRGB

} CrystalFlightLitVertex;

//	Each tile is a SIMD4<Float16> whose first three components
//	give the translation, and whose fourth component
//	is set to 1.0 if the tile should be drawn,
//	or to 0.0 if it should be culled.
//
//	Note #1:  Even though we use Float32 for vertex coordinates,
//	it's fine to use Float16 for the translation. If a tile
//	ends up slightly off relative to one of its neighbors,
//	we'll never notice, because where a bond in one tile meets
//	an atom in a neighboring tile, the atom completely hides the junction.
//
//	Note #2:  The Metal Shading Language's "half3" and "half4"
//	data types both require 8 bytes, so the that fourth coordinate
//	we're using "comes along for free".
//
typedef faux_simd_half4 CrystalFlightTile;

typedef struct {

	uint			itsTileCount;
	simd_short3		itsGridLowerLimits;
	simd_ushort3	itsGridDimensions;
	simd_float3x3	itsGridTranslations;
	simd_float4x3	itsOutwardUnitNormals;	//	outward unit normals to view frustum
	simd_float3		itsBoundingSphereCenter;
	float			itsBoundingSphereRadius;
	float			itsAdjustedHorizonRadius;
	faux_simd_half3	itsDistanceCutoffs;	//	cutoffs for progressively finer levels of detail
	
} CrystalFlightTilingParameters;

#define gCompactionThreadgroupMaxHeight	1024u

#endif	//	CrystalFlightGPUDefinitions_h
