//	CrystalFlightHelpView.swift
//
//	© 2023 by Jeff Weeks
//	See TermsOfUse.txt

import SwiftUI


struct CrystalFlightHelpView: View {

	var body: some View {

		VStack(alignment: .leading, spacing: 12) {
			
			CrystalFlightHelpItem(
				what: "Steer-what",
				how: "Steer-how",
				altHow: "Steer-altHow")

			CrystalFlightHelpItem(
				what: "Speed-what",
				how: "Speed-how",
				altHow: nil)

			CrystalFlightHelpItem(
				what: "Pause-what",
				how: "Pause-how",
				altHow: nil)

			CrystalFlightHelpItem(
				what: "Sidle-what",
				how: "Sidle-how",
				altHow: nil)

			CrystalFlightHelpItem(
				what: "Forward-what",
				how: "Forward-how",
				altHow: nil)

			CrystalFlightHelpItem(
				what: "Roll-what",
				how: "Roll-how",
				altHow: nil)
		}
		.padding(geometryGamesPanelPadding)
		.background(Color(.displayP3, red: 0.750, green: 0.875, blue: 1.000, opacity: 1.0))
		.cornerRadius(geometryGamesCornerRadius)
	}
}

struct CrystalFlightHelpItem: View {
	
	let what: LocalizedStringKey
	let how: LocalizedStringKey
	let altHow: LocalizedStringKey?
	
	var body: some View {

		VStack(alignment: .leading) {

			Text(what)
				.font(.title3)
				.italic()
				.foregroundColor(Color(.displayP3,
										red:   0.00,
										green: 0.25,
										blue:  0.50,
										opacity: 1.0))

			Text(how)
				.padding(EdgeInsets(
							top: 0.0,
							leading: 16.0,
							bottom: 0.0,
							trailing: 0.0))
				.font(.body)
				.foregroundColor(.black)

			if let theAltHow = altHow {
	
				Text(theAltHow)
					.padding(EdgeInsets(
								top: 0.0,
								leading: 20.0,
								bottom: 0.0,
								trailing: 0.0))
					.font(.footnote)
					.foregroundColor(.gray)
			}
		}
		//	Counterintuitively, .lineLimit(nil) no longer works
		//	to allow multiline Text views, but .fixedSize(…) does.
		//	Note:  SwiftUI's .lineLimit(nil) but is deterministic
		//	but inconsistent -- you might have three lines that
		//	get laid out correctly, but then when you add a fourth,
		//	the first line suddenly starts getting truncated.
		.fixedSize(horizontal: false, vertical: true)
	}
}


struct CrystalFlightHelpView_Previews: PreviewProvider {
	static var previews: some View {

		CrystalFlightHelpView()
			.environment(\.locale, .init(identifier: "pt"))
			.previewDevice(PreviewDevice(rawValue: "iPhone SE (2nd generation)"))
	}
}
