//	CrystalFlightScreenshots.swift
//
//	© 2023 by Jeff Weeks
//	See TermsOfUse.txt

import SwiftUI


//	Set gGetScreenshotOrientations to true while looking
//	for an aesthetically pleasing quaternion for itsUserOrientation
//	for each type of screenshot, copy your preferred quaternion
//	into prepareForAppStoreScreenshot(), and then set gGetScreenshotOrientations
//	back to false before making the screenshots.
//#warning("disable gGetScreenshotOrientations")
let gGetScreenshotOrientations = false

//	To make the single Crystal Flight screenshot for its Geometry Games web page,
//	set gMakeWebSiteScreenshot to true.
//#warning("disable gMakeWebSiteScreenshot")
let gMakeWebSiteScreenshot = false

//	To make app store screenshots, set gMakeAppStoreScreenshots to true.
//#warning("disable gMakeAppStoreScreenshots")
let gMakeAppStoreScreenshots = false


private let gAppStoreScreenshotPanels: [PanelType] = [
	.noPanel,		//	CrystalHalite
	.crystalPanel,	//	CrystalIceHexagonal
	.helpPanel,		//	CrystalDiamond
	.optionsPanel,	//	CrystalQuartz
	.exportPanel	//	CrystalDiamond
]
let gNumAppStoreScreenshots = gAppStoreScreenshotPanels.count


func prepareForWebSiteScreenshot(
	modelData: CrystalFlightModel,
	activePanel: Binding<PanelType>
) {

	modelData.itsCrystal = .iceHexagonal
	modelData.itsHorizonRadius = 64.0	//	ångströms
	modelData.itsAtomicRadiusFactor = 1.0
	modelData.itsUserPosition
		= SIMD3<Double>(-1.56687503583540111,  0.63290160095650760,  0.36060143336120348)
	modelData.itsUserOrientation = simd_quatd(
		real: 0.9666077744053256,
		imag: SIMD3<Double>(0.21968148055252099, -0.13034807694028444, -0.020465492894264463))

	modelData.itsDisplayChangeCount += 1

	activePanel.wrappedValue = .crystalPanel
}


func prepareForAppStoreScreenshot(
	_ screenshotIndex: Int,
	modelData: CrystalFlightModel,
	activePanel: Binding<PanelType>
) {

	switch screenshotIndex {
	
	case 0:
		modelData.itsCrystal = .iceHexagonal
		modelData.itsHorizonRadius = 32.0	//	ångströms
		modelData.itsAtomicRadiusFactor = 1.0
		modelData.itsUserPosition
			= SIMD3<Double>(-1.56687503583540111,  0.63290160095650760,  0.36060143336120348)
		modelData.itsUserOrientation = simd_quatd(
			real: 0.9666077744053256,
			imag: SIMD3<Double>(0.21968148055252099, -0.13034807694028444, -0.020465492894264463))

	case 1:
		modelData.itsCrystal = .halite
		modelData.itsHorizonRadius = 32.0	//	ångströms
		modelData.itsAtomicRadiusFactor = 0.4
		modelData.itsUserPosition
			= SIMD3<Double>(-1.41005000000000003, -1.41005000000000003,  0.00000000000000000)
		modelData.itsUserOrientation = simd_quatd(real: 1.0, imag: SIMD3<Double>(0.0, 0.0, 0.0))

	case 2:
		modelData.itsCrystal = .quartz
		modelData.itsHorizonRadius = 32.0	//	ångströms
		modelData.itsAtomicRadiusFactor = 0.8
		modelData.itsUserPosition = SIMD3<Double>( 1.51430170375740314, -2.01392357471623207,  0.56452163635345454)
		modelData.itsUserOrientation = simd_quatd(
			real: -0.1625894773109881,
			imag: SIMD3<Double>(0.7133802273989066, 0.6435751925417332, -0.22464257069597277))

	case 3:
		modelData.itsCrystal = .diamond
		modelData.itsHorizonRadius = 32.0	//	ångströms
		modelData.itsAtomicRadiusFactor = 0.5
		modelData.itsUserPosition = SIMD3<Double>( 0.89997643076039857, -0.09881104831823119, -0.52715865297904008)
		modelData.itsUserOrientation = simd_quatd(
			real: 0.7536680792233464,
			imag: SIMD3<Double>(0.5662071404842738, -0.09665220917099256, 0.3194561799164011))

	case 4:
		modelData.itsCrystal = .diamond
		modelData.itsHorizonRadius = 32.0	//	ångströms
		modelData.itsAtomicRadiusFactor = 0.25
		modelData.itsUserPosition = SIMD3<Double>( 0.89997643076039857, -0.09881104831823119, -0.52715865297904008)
		modelData.itsUserOrientation = simd_quatd(
			real: 0.7536680792233464,
			imag: SIMD3<Double>(0.5662071404842738, -0.09665220917099256, 0.3194561799164011))

	default:
		modelData.itsCrystal = .iceHexagonal
		modelData.itsHorizonRadius = 8.0	//	ångströms
		modelData.itsAtomicRadiusFactor = gMinAtomicRadiusFactor
		modelData.itsUserPosition = SIMD3<Double>.zero
		modelData.itsUserOrientation = simd_quatd(real: 1.0, imag: SIMD3<Double>(0.0, 0.0, 0.0))
	}
	
	modelData.itsDisplayChangeCount += 1

	activePanel.wrappedValue = gAppStoreScreenshotPanels[screenshotIndex]
}
